@@ -18,6 +18,7 @@ import contacts as ContactH
 import gobject
 import dbus
 import mimetypes
+from gnome import gnomevfs
 
 class fMMS_GUI(hildon.Program):
 
@@ -417,18 +418,28 @@ class fMMS_GUI(hildon.Program):
 	
 	""" action on redl contextmenu click """
 	def liststore_redl_clicked(self, widget):
-		(model, miter) = self.treeview.get_selection().get_selected()
-		# the 4th value is the filename (start counting at 0)
-		filename = model.get_value(miter, 3)
-		print "redownloading", filename
-		try:
-			self.delete_mms(filename)
-			self._get_mms_from_push(self._pushdir + "/" + filename)
-			self.show_mms(self.treeview, model.get_path(miter))
-		except Exception, e:
-			print e, e.args
-			banner = hildon.hildon_banner_show_information(self.window, "", "fMMS: Operation failed")
-		hildon.hildon_gtk_window_set_progress_indicator(self.window, 0)
+		dialog = gtk.Dialog()
+		dialog.set_title("WARNING")
+		dialog.add_button(gtk.STOCK_YES, 1)
+		dialog.add_button(gtk.STOCK_NO, 0)
+		label = gtk.Label("If the message is no longer on your MMSC,\n the message will be lost. Continue?")
+		dialog.vbox.add(label)
+		dialog.show_all()
+		ret = dialog.run()
+		if ret == 1:
+			(model, miter) = self.treeview.get_selection().get_selected()
+			# the 4th value is the filename (start counting at 0)
+			filename = model.get_value(miter, 3)
+			print "redownloading", filename
+			try:
+				self.delete_mms(filename)
+				self._get_mms_from_push(self._pushdir + "/" + filename)
+				self.show_mms(self.treeview, model.get_path(miter))
+			except Exception, e:
+				print e, e.args
+				banner = hildon.hildon_banner_show_information(self.window, "", "fMMS: Operation failed")
+		dialog.destroy()
+		return
 
 	""" long press on image creates this """
 	def liststore_mms_menu(self):
@@ -628,14 +639,27 @@ class fMMS_GUI(hildon.Program):
 		textview.set_buffer(textbuffer)
 		container.add(textview)
 		
-	""" action on click on image """
+	""" action on click on image/button """
 	def mms_img_clicked(self, widget, data):
 		print widget, data
 		path = "file://" + data
-		print path
-		rpc = osso.Rpc(self.osso_c)
-		rpc.rpc_run("com.nokia.image_viewer", "/com/nokia/image_viewer", "com.nokia.image_viewer", "mime_open", (str, path))
-		
+		# gnomevfs seems to be better than mimetype when guessing mimetype for us
+		file_mimetype = gnomevfs.get_mime_type(path)
+		file_mimetype = None
+		if file_mimetype != None:
+			if file_mimetype.startswith("video") or file_mimetype.startswith("audio"):
+				rpc = osso.Rpc(self.osso_c)
+				rpc.rpc_run("com.nokia.mediaplayer", "/com/nokia/mediaplayer", "com.nokia.mediaplayer", "mime_open", (str, path))	
+			elif file_mimetype.starts_with("image"):
+				rpc = osso.Rpc(self.osso_c)
+				rpc.rpc_run("com.nokia.image_viewer", "/com/nokia/image_viewer", "com.nokia.image_viewer", "mime_open", (str, path))
+		else:
+			# TODO: how to solve this?
+			# move .mms to ~/MyDocs? change button to copy file to ~/MyDocs?
+			#rpc = osso.Rpc(self.osso_c)
+			#path = os.path.dirname(path).replace("file://", "")
+			print path
+			#rpc.rpc_run("com.nokia.osso_filemanager", "/com/nokia/osso_filemanager", "com.nokia.osso_filemanager", "open_folder", (str, path))
 
 
 	""" long press on image creates this """
