@@ -9,7 +9,6 @@ import sys
 from mms import message
 from mms.message import MMSMessage
 from mms import mms_pdu
-from mms.WSP import WSP
 import os
 import dbus
 import urllib2
@@ -65,22 +64,34 @@ class PushHandler:
 			#print "WSPPAYLOAD: ", wsp_payload
 
 		data = []
-		for d in wsp_header:
-			data.append(int(d))	
+		# We didn't really want this, as we decode as MMS
+		# pretty bad idea to decode WSP header as MMS, I know :(
+		#for d in wsp_header:
+		#	data.append(int(d))	
 		for d in wsp_payload:
-			data.append(int(d)) 
+			data.append(int(d))
+
+		print "decoding..."
 		# decode the push message
 		decoder = mms_pdu.MMSDecoder()
 		#print data
 		wspdata = decoder.decodeCustom(data)
 		# extract some data we want to use
-		sndr = wspdata["From"]
-		url = wspdata["Content-Location"]
-		trans_id = wspdata["Transaction-Id"]
-		trans_id = str(trans_id)
-		print trans_id
-		print sndr
-		print url
+		try:
+			url = wspdata["Content-Location"]
+			print "content-location:", url
+			trans_id = wspdata["Transaction-Id"]
+			trans_id = str(trans_id)
+			print "transid:", trans_id
+			sndr = wspdata["From"]
+			print "Sender:", sndr
+		except Exception, e:
+			print "exception on values from decode:", e, e.args
+		
+		if sndr == None:
+			print "No sender... Setting to unknown"
+			sndr = "Unknown"
+		
 		# save to file
 		self._save_push_message(trans_id, wspdata)
 		# notify the user we got a new message
@@ -149,6 +160,9 @@ class PushHandler:
 			time.sleep(7)
 			con.connect()
 			
+			### send the m-notifyresp-ind first
+			notifyresp = self._send_notify_resp(transaction)
+			print "notifyresp sent"
 			# this seems to work like it should
 			# TODO: dont hardcode timeout
 			# god it would be nice with python2.6 for timeout= in urllib2
@@ -172,7 +186,11 @@ class PushHandler:
 			
 			if(_DBG):
 				print "fetched ", location, " and wrote to file", fp
-			con.disconnect()	
+				
+			# send acknowledge we got it ok
+			ack = self._send_acknowledge(transaction)
+			print "ack sent"
+			con.disconnect()
 			
 		except Exception, e:
 			print "fetch failed."
@@ -186,6 +204,31 @@ class PushHandler:
 			raise
 		return dirname
 
+	def _send_notify_resp(self, transid):
+		mms = MMSMessage(True)
+		mms.headers['Message-Type'] = "m-notifyresp-ind"
+		mms.headers['Transaction-Id'] = transid
+		mms.headers['MMS-Version'] = "1.3"
+		mms.headers['Status'] = "Deferred"
+		
+		print "setting up notify sender"
+		sender = MMSSender(customMMS=True)
+		print "sending notify..."
+		out = sender.sendMMS(mms)
+		print "m-notifyresp-ind:", out
+		return out
+		
+	def _send_acknowledge(self, transid):
+		mms = MMSMessage(True)
+		mms.headers['Message-Type'] = "m-acknowledge-ind"
+		mms.headers['Transaction-Id'] = transid
+		mms.headers['MMS-Version'] = "1.3"
+		
+		ack = MMSSender(customMMS=True)
+		out = ack.sendMMS(mms)
+		print "m-acknowledge-ind:", out
+		return out
+
 	""" decodes 'message'-file in path """
 	def decodeMMS(self, path):
 		# Decode the specified file
@@ -248,16 +291,19 @@ class ConnectToAPN:
     	    
 """ class for sending an mms """    	    
 class MMSSender:
-	def __init__(self, number, subject, message, attachment, sender):
+	def __init__(self, number=None, subject=None, message=None, attachment=None, sender=None, customMMS=None):
 		print "GOT SENDER:", sender
-		self.number = number
-		self.subject = subject
-		self.message = message
-		self.attachment = attachment
-		self._mms = None
-		self._sender = sender
-		self.createMMS()
+		print "customMMS:", customMMS
+		self.customMMS = customMMS
 		self.config = fMMSconf.fMMS_config()
+		if customMMS == None:
+			self.number = number
+			self.subject = subject
+			self.message = message
+			self.attachment = attachment
+			self._mms = None
+			self._sender = sender
+			self.createMMS()
 	    
 	def createMMS(self):
 		slide = message.MMSMessagePage()
@@ -271,9 +317,13 @@ class MMSSender:
 		self._mms.headers['From'] = str(self._sender) + '/TYPE=PLMN'
 		self._mms.addPage(slide)
 	
-	def sendMMS(self):
+	def sendMMS(self, customData=None):
+		if customData != None:
+			print "using custom mms"
+			self._mms = customData
+			
 		mmsc = self.config.get_mmsc()
-
+		
 		(proxyurl, proxyport) = self.config.get_proxy_from_apn()
 		mms = self._mms.encode()
 		
